import { world, system, EntityEquippableComponent, EquipmentSlot, ItemStack, EnchantmentType, EffectType, EntityHealthComponent, Player, EntityComponentTypes } from '@minecraft/server';
import { items } from './itemRegister';


world.afterEvents.entityHurt.subscribe((event) => {
    const entity = event.hurtEntity;

    // Check if the entity is a player
    if (entity?.typeId === "minecraft:player") {
        world.playSound("hit", entity.location)
		if (entity.getEffect("minecraft:poison") || entity.getEffect("minecraft:wither") || entity.getComponent(EntityComponentTypes.OnFire)) {
			entity.applyKnockback(0,0,0,0);
		}
    }
});


export let playerdata = {};
system.runInterval(() => {
  for (const player of world.getPlayers()) {
    const isSprinting = player.isSprinting;
    const isOnGround = player.isOnGround;
    system.runTimeout(() => {
      playerdata[player.id] = {
        isSprinting,
        isOnGround,
      };
    }, 2);
  }
}, 2);

world.afterEvents.projectileHitEntity.subscribe(evd => {
  const { source: attacker, projectile } = evd;
  const victim = evd.getEntityHit().entity;
  if (!attacker) {return}

  if (projectile?.typeId === "minecraft:arrow") {
    attacker.runCommand("playsound random.orb @s");
  }
});





const cooldowns = new Map();

const cooldownPercentages = new Map();
const lastSelectedItem = new Map();

const activeCooldowns = new Set(); // Track players with active cooldowns




function startCooldown(player, cooldownType, cooldownDuration) {
    const cooldownStart = Date.now();  // When the cooldown starts
    const cooldownEnd = cooldownStart + cooldownDuration * 1000;  // When the cooldown ends
    const playerCooldowns = cooldowns.get(player.id) || {};

    // Save both cooldownStart and cooldownEnd for the player
    playerCooldowns[cooldownType] = { cooldownStart, cooldownEnd };
    cooldowns.set(player.id, playerCooldowns);
    activeCooldowns.add(player.id);

    // Initialize cooldown percentage to 0%
    player.addTag("usingItem");

    // Remove the "usingItem" tag after 1 second (20 ticks)
    system.runTimeout(() => {
        player.removeTag("usingItem");
    }, 20);
}


function getCooldownPercent(player, cooldownType) {
    const playerCooldowns = cooldowns.get(player.id) || {};
    const cooldownData = playerCooldowns[cooldownType];

    if (cooldownData) {
        const remainingTime = Math.max(cooldownData.cooldownEnd - Date.now(), 0);
        const cooldownPercent = Math.max(100 - (remainingTime / (cooldownData.cooldownEnd - cooldownData.cooldownStart)) * 100, 0);  // Flipped percentage
        return cooldownPercent;
    }

    return 100;  // No cooldown means fully available
}




function getPlayerCooldownPercent(playerId) {
    return cooldownPercentages.get(playerId) || 100; // Default to 100 if not found
}

system.runInterval(() => {
    activeCooldowns.forEach((playerId) => {
        const player = world.getAllPlayers().find(p => p.id === playerId);
        if (!player) {
            activeCooldowns.delete(playerId); // Remove disconnected players
            return;
        }

        const cooldownPercent = getCooldownPercent(player, "sweep");
        // Update the cooldown HUD only if the cooldown percent changes
        const lastPercent = player.getDynamicProperty("lastCooldownPercent") || 0;
        if (Math.floor(cooldownPercent) !== Math.floor(lastPercent)) {
            // Set the title (this could be a visual progress bar, such as an icon or text)
            player.onScreenDisplay.setTitle(`_sweepnslash:crs:f:${cooldownPercent.toFixed(0)}%`, {
                fadeInDuration: 0,
                fadeOutDuration: 0,
                stayDuration: 0,
            });

            player.setDynamicProperty("lastCooldownPercent", cooldownPercent);
        }

        // Remove player from active list if cooldown is complete
        if (cooldownPercent >= 100) {
			player.onScreenDisplay.setTitle(`_sweepnslash:non`, {
                fadeInDuration: 0,
                fadeOutDuration: 0,
                stayDuration: 0,
            });
			
            activeCooldowns.delete(playerId);
        }
    });
}, 1); // Runs every tick (1/20th of a second)



function activateCooldown(player) {
  const inventory = player.getComponent('minecraft:inventory')?.container;
  const selectedItem = inventory?.getItem(player.selectedSlotIndex);
  
  // Debug log for selected item
  

  if (selectedItem) {
    let cooldownApplied = false;

    // Check the item register for a match
    for (const item of items) {
		if (player.hasTag("sword")) {
			return;
		  }
     if (selectedItem && selectedItem.typeId.includes("mace")) {
		handleCooldowns(player, selectedItem, 2.2, 'sweep');
		return;
	  }
      if (selectedItem.typeId.includes(item.item)) {
        cooldownApplied = true;
        for (const specificItem of item.specific) {

          if (
            specificItem.type === 'all' || 
            (Array.isArray(specificItem.type) && specificItem.type.some(type => selectedItem.typeId.includes(type)))
          ) {
  
            handleCooldowns(player, selectedItem, specificItem.duration, 'sweep');
          }
        }
        break;
      }
    }
	
    // If no match was found in the register, check item tags
    if (!cooldownApplied) {

      if (selectedItem.hasTag('is_sword')) {
      
        handleCooldowns(player, selectedItem, 0.5, 'sweep');
      } else if (selectedItem.hasTag('is_axe')) {
        handleCooldowns(player, selectedItem, 1, 'sweep');
      } else {
        handleCooldowns(player, selectedItem, 0.4, 'sweep'); // Default cooldown
      }
    }
  } else {
    // No item selected, apply default cooldown
    handleCooldowns(player, null, 0.4, 'sweep');
  }
}


function handleCooldowns(player, item, cooldownDuration, cooldownType) {
    const cooldownKey = `cd:${cooldownType}Timer`;

    // Retrieve the haste and mining speed bonuses
    const hasteSpeed = getHasteSpeed(player);
    const miningSpeed = getMiningSpeed(player);

    // Adjust cooldown duration with haste and mining speed
    const adjustedCooldownDuration = cooldownDuration * (1 - (hasteSpeed * 0.5) + (miningSpeed * 0.5));
    startCooldown(player, cooldownType, adjustedCooldownDuration);
}


let lastMainHandItem = null; // Store the last known item in the player's main hand

// Start the interval to periodically check for item changes
system.runInterval(() => {
    world.getAllPlayers().forEach(player => {
        if (!player) return;

        const inv = player.getComponent("inventory")?.container;
        const item = inv?.getItem(player.selectedSlotIndex);
        const lastItem = player.getDynamicProperty("last_item_held") || "";

        if (item) {
            // Compare the item typeId properly
            if (item.typeId === "minecraft:mace" && (lastItem.includes("sword") || lastItem.includes("_axe"))) {
                player.addTag("sword");  // Add "sword" tag if the condition is met
				system.runTimeout(() => {
					player.removeTag("sword");
				}, 10);
            }

            // If the item has changed, trigger cooldown
            if (item.typeId !== lastItem) {
                activateCooldown(player);
                player.setDynamicProperty("last_item_held", item.typeId);  // Update the last item held
            }
        } else {
            // If no item is in hand, clear the stored item
            player.setDynamicProperty("last_item_held", "");
        }
    });
}, 5);  // Runs every 5 ticks


// Fire Arrows make Minecarts explode when near them.
system.runInterval(function cartExplode() {
    world.getAllPlayers().forEach((player) => {
        const arrows = player.dimension.getEntities({ type: 'arrow' })
        for (const arrow of arrows) {
            const arrowfire = arrow.getComponent(EntityComponentTypes.OnFire)
            if (arrowfire) {
                arrow.runCommandAsync('event entity @e[type=tnt_minecart,x=~-4,y=~0.3,z=~-4,dx=6,dy=5,dz=5,c=1] minecraft:on_instant_prime')
            }
        }
    })
})


const unicodes = ['', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', ''];


function spawnCriticalHitParticles(dimension, location) {
  const particlePos = { x: location.x, y: location.y + 0.625, z: location.z };
  for (let i = 0; i < 3; i++) {
    dimension.spawnParticle("dave:critical_hit_custom", particlePos);
  }
}



// Subscribe to item use event (when the item is used)
world.afterEvents.itemUse.subscribe(event => {
    const player = event.source;
    const itemUsed = event.itemStack;
	
	 if (player && itemUsed) {
        // Add the tag to indicate the player is using a food item
        player.addTag("usingItem");

        // Optional: you can set a cooldown or some other effect here
    }
});

// Subscribe to item stop use event (when the player stops using the item)
world.afterEvents.itemStopUse.subscribe(event => {
    const player = event.source;

    if (player) {
        // Remove the tag when the player stops using the item
        player.removeTag("usingItem");
    }
});





function isBlockingWithShield(player) {
    // Get the player's inventory and check the main hand item
    const inventory = player.getComponent("minecraft:inventory")?.container;
    const mainHandItem = inventory?.getItem(player.selectedSlotIndex);

    // Get the offhand item (where shields are usually held when blocking)
    const offhandItem = player.getComponent("minecraft:equippable")?.getEquipment(EquipmentSlot.Offhand);

    // Check if the player is sneaking (crouching) and has a shield in either hand
    return player.isSneaking && (
        (mainHandItem && mainHandItem?.typeId === "minecraft:shield") || 
        (offhandItem && offhandItem?.typeId === "minecraft:shield")
    );
}


function getHasteSpeed(entity) {
    // Retrieve the haste effect from the entity
    const hasteEffect = entity.getEffect("minecraft:haste");

    // Check if the haste effect is present
    if (!hasteEffect) {
        return 0; // No haste effect applied
    }

    // Retrieve the amplifier (Haste I = amplifier 0, Haste II = amplifier 1, etc.)
    const amplifier = hasteEffect.amplifier;

    // Calculate the speed bonus using a mathematical formula
    const hasteSpeed = 0.03 * (amplifier + 1); // +1 because amplifier 0 is Haste I, etc.

    // Return the calculated haste speed
    return hasteSpeed;
}

function getMiningSpeed(entity) {
    // Retrieve the haste effect from the entity
    const miningEffect = entity.getEffect("minecraft:mining_fatigue");

    // Check if the haste effect is present
    if (!miningEffect) {
        return 0; // No haste effect applied
    }

    // Retrieve the amplifier (Haste I = amplifier 0, Haste II = amplifier 1, etc.)
    const amplifier = miningEffect.amplifier;

    // Calculate the speed bonus using a mathematical formula
    const miningSpeed = 0.03 * (amplifier + 1); // +1 because amplifier 0 is Haste I, etc.

    // Return the calculated haste speed
    return miningSpeed;
}


// Use a map to track selected items for each player
const intervalIds = new Map();


let intervalRunning = true;


function applyKnockbackWithDelay(attacker, direction) { 
    attacker.applyKnockback(direction.x, direction.z, 0.2, -0.0);
}
function applyhit(attacker, direction) { 
    attacker.applyKnockback(direction.x, direction.z, 0.2, 0.2);
}
function applyantihit(attacker, direction) { 
    attacker.applyKnockback(direction.x, direction.z, 0, 0);
}

const vec2YSub = (a, b) => {return a.y - b.y}

const recentHits = new Map();

world.afterEvents.entityHitEntity.subscribe(async (event) => {
    const attacker = event.damagingEntity;
    const target = event.hitEntity;
    const attackerViewDirection = attacker.getViewDirection();
	const PRot = attacker.getRotation();
    const DRot = target.getRotation();
    const YDeferent = vec2YSub(PRot, DRot);
    // Ensure the attacker is a player and both entities are valid
    if (!attacker || !target) {
		return;
	}
	if (attacker.typeId !== "minecraft:player" && (isBlockingWithShield(target) && !(YDeferent <= 90 && YDeferent >= -90) && !target.hasTag("usingItem") && !target.hasTag("shield_b"))) {
        // Apply knockback
        applyantihit(attacker, attackerViewDirection);
		return;
    }
	if (target.typeId == "minecraft:boat") {
		target.applyDamage(100, {
        cause: "none",
        source: attacker,
		});
		return;
	}
    // Get the inventory component of the attacker
    const inventory = attacker.getComponent("minecraft:inventory")?.container;
    if (!inventory) return;

    // Get the main hand item (assumed selectedSlotIndex is the main hand slot)
    const mainHandItem = inventory.getItem(attacker.selectedSlotIndex);
    // Check shield blocking conditions
    if (isBlockingWithShield(target) && !(YDeferent <= 90 && YDeferent >= -90) && !target.hasTag("usingItem") && !target.hasTag("shield_b")) {
        
        // Apply knockback
        attacker.applyKnockback(0,0,0,0);

        // If holding an axe, add and remove shield_b tag with delay
        if (mainHandItem && mainHandItem.typeId.includes("axe")) {
            target.addTag("shield_b");
            system.runTimeout(() => {
                target.removeTag("shield_b");
                target.runCommandAsync("playsound random.orb @s"); // Confirm tag removal with a sound
            }, 100);
            // Wait for 5 seconds (5000 milliseconds) then remove the "shield_b" tag
          
			return;
        }
        
        return;
    }
    // Ensure both the damaging entity and hit entity exist
    if (attacker && target) {
        recentHits.set(target.id, attacker);
    }
	
	handleHit(attacker, target);
	activateCooldown(attacker);
});

// For air swinging and parsing item stats from other addons
system.afterEvents.scriptEventReceive.subscribe((event) => {
    const { id, sourceEntity } = event;
	const cooldownPercent = getCooldownPercent(sourceEntity, "sweep");
    // Check if the script event ID matches
    if (id === "bf:attack" && cooldownPercent > 50) {
        // Ensure the sourceEntity exists and is a player
        if (sourceEntity?.typeId === "minecraft:player") {
            activateCooldown(sourceEntity); // Pass the player entity
        }
    }
});


// Function to get bonus damage based on the strength effect's level
function getStrengthDamage(entity) {
  // Retrieve the strength effect from the entity
  const strengthEffect = entity.getEffect("minecraft:strength");

  // Check if the strength effect is present
  if (!strengthEffect) {
    return 0; // No strength effect applied
  }

  // Retrieve the amplifier (Strength I = amplifier 0, Strength II = amplifier 1, etc.)
  const amplifier = strengthEffect.amplifier;

  // Calculate the bonus damage using a mathematical formula
  const strengthDamage = 3 * (amplifier + 1); // +1 because amplifier 0 is Strength I, etc.

  // Return the calculated strength damage
  return strengthDamage;
}

// Function to get bonus damage based on the strength effect's level
function getWeaknessDamage(entity) {
  // Retrieve the strength effect from the entity
  const weaknessEffect = entity.getEffect("minecraft:weakness");

  // Check if the strength effect is present
  if (!weaknessEffect) {
    return 0; // No strength effect applied
  }

  // Retrieve the amplifier (Strength I = amplifier 0, Strength II = amplifier 1, etc.)
  const amplifier = weaknessEffect.amplifier;

  // Calculate the bonus damage using a mathematical formula
  const weaknessDamage = 3 * (amplifier + 1); // +1 because amplifier 0 is Strength I, etc.

  // Return the calculated strength damage
  return weaknessDamage;
}


// Function to get sharpness damage from the item based on its enchantments
function getSharpnessDamage(item) {
  // Get the enchantable component of the item
  const enchantableComponent = item.getComponent("enchantable");
  
  // Check if the component is present
  if (!enchantableComponent) {
    return 0; // No enchantments available
  }

  // Retrieve the sharpness enchantment level
  const sharpnessLevel = enchantableComponent.getEnchantment(new EnchantmentType("sharpness"))?.level || 0;

  // Calculate sharpness damage based on the level
  // Adjust these values according to your design for sharpness damage scaling
  const sharpnessDamageMap = {
    1: 1,   // Sharpness I
    2: 1.5, // Sharpness II
    3: 2,   // Sharpness III
    4: 2.5,   // Sharpness IV
    5: 3    // Sharpness V
  };

  // Return the sharpness damage based on the level found
  return sharpnessDamageMap[sharpnessLevel] || 0; // Return 0 if no sharpness
}


function handleHit(attacker, hitEntity) {
  const attackerViewDirection = attacker.getViewDirection();
  const cooldownPercent = getCooldownPercent(attacker, "sweep"); // Use cooldown data tied to the attacker

  if (cooldownPercent < 55) {
	hitEntity.applyKnockback(0,0,0,0);
	return;
  } else {
	hitEntity.applyKnockback(
		-Math.sin(attacker.getRotation().y * (Math.PI / 180)),
		Math.cos(attacker.getRotation().y * (Math.PI / 180)),
		0.3,  // Horizontal knockback strength
		0.3 // Vertical knockback
	);
	const hitLocation = hitEntity.location;
		const dimension = hitEntity.dimension;
		const particlePos = {
				x: hitLocation.x,
				y: hitLocation.y + 1,
				z: hitLocation.z,
			};
	dimension.spawnParticle("bf:damage_indicator_emitter", particlePos);
    world.playSound("hit", hitEntity.location);
  }
  // Smoothly interpolate critvolplus and critvolminus based on cooldownPercent
  let critvolplus = 1;
  let critvolminus = 1;

  if (cooldownPercent >= 55 && cooldownPercent <= 100) {
    const t = (cooldownPercent - 75) / 25; // Normalize to a range of 0 (at 75%) to 1 (at 100%)
    critvolminus = 1.4 - 0.4 * t; // Linearly interpolate from 1.4 to 1
    critvolplus = 1.4 + 0.4 * t; // Linearly interpolate from 1.4 to 1
  }
	const inventory = attacker.getComponent("minecraft:inventory")?.container;
	const selectedItem = inventory?.getItem(attacker.selectedSlotIndex);
	if (!selectedItem) {return;}
	const isCritical = ((!playerdata[attacker.id]?.isOnGround && !attacker.up && !attacker.isInWater) || (attacker.isInWater && !playerdata[attacker.id]?.isSprinting)) && cooldownPercent > 55;

	const isKnockBack = (playerdata[attacker.id]?.isSprinting || selectedItem.typeId.includes("axe")) && (attacker.up || playerdata[attacker.id]?.isOnGround) && cooldownPercent > 55;
	
	const sharpness = getSharpnessDamage(selectedItem)
	if (cooldownPercent > 55 && sharpness > 0) {
		const hitLocation = hitEntity.location;
		const dimension = hitEntity.dimension;
		const particlePos = {
				x: hitLocation.x,
				y: hitLocation.y + 1,
				z: hitLocation.z,
			};
		dimension.spawnParticle("bf:enchanted_hit_emitter", particlePos);
	}
	if (isCritical) {
		const hitLocation = hitEntity.location;
		if (hitLocation) {
			// Get the dimension from the hit entity
			const dimension = hitEntity.dimension;
			if (selectedItem.typeId.includes("_axe")) {critvolminus = critvolminus * 0.9; critvolplus = critvolplus * 2;}
			// Play critical hit sound
			world.playSound("critical_hit", hitLocation, {
				pitch: critvolminus,
				volume: critvolplus,
			});

			const particlePos = {
				x: hitLocation.x,
				y: hitLocation.y + 2,
				z: hitLocation.z,
			};
			dimension.spawnParticle("bf:critical_hit_emitter", particlePos);
		}
	} else if (isKnockBack) {
		hitEntity.applyKnockback(
			-Math.sin(attacker.getRotation().y * (Math.PI / 180)),
			Math.cos(attacker.getRotation().y * (Math.PI / 180)),
			0.8,  // Horizontal knockback strength
			0.35 // Vertical knockback
		);

		// Play knockback sound
		world.playSound("knockback", hitEntity.location);
	} else if (selectedItem.typeId.includes("sword") && cooldownPercent >= 55) {
		const attackDimension = world.getDimension(attacker.dimension.id);
		const attackDirection = attacker.getRotation().y * (Math.PI / 180);
		const offsetX = -Math.sin(attackDirection) * 2;
		const offsetZ = Math.cos(attackDirection) * 2;
		const particlePosition = {
		  x: attacker.location.x + offsetX,
		  y: attacker.location.y + 0.2,
		  z: attacker.location.z + offsetZ
		};

		attackDimension.spawnParticle("dave:sweep", particlePosition);
		world.playSound("sweep", particlePosition);
		
		attackDimension.getEntities({
		  location: hitEntity.location,
		  maxDistance: 1.5
		}).forEach(entity => {
		  if (entity.id === attacker.id || entity.id === hitEntity.id) return;

		  const damage = 3 * (Math.random() * 0.25 + 0.5);
		  entity.applyDamage(damage, {
			cause: "none",
			damagingEntity: attacker
		  });
		});
	}

  let caldamage = 1;

  if (selectedItem) {
    const itemId = selectedItem.typeId;
    const matchedItem = items.find((item) => item.typeId === itemId);
    if (matchedItem) {
      const { extraDamage: damage, cooldown } = matchedItem;
		
      // Add sharpness, strength, and weakness contributions
      caldamage =
        damage +
        getSharpnessDamage(selectedItem) +
        getStrengthDamage(attacker) -
        getWeaknessDamage(attacker);

      // Ensure thresholds and outputs are properly aligned
      const thresholds = [
        0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60,
        65, 70, 75, 80, 85, 90, 95, 100,
      ];

      const outputs = [
        0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3125, 0.3445,
        0.3805, 0.4205, 0.4645, 0.5125, 0.5645, 0.6205, 0.6805, 0.7445, 0.8125,
        0.8845, 0.9605, 1.0,
      ];

      // Adjust caldamage based on cooldownPercent
      for (let i = 0; i < thresholds.length; i++) {
        if (cooldownPercent <= thresholds[i]) {
          caldamage *= outputs[i];
          break;
        }
      }

      if (isCritical) {
        caldamage *= 1.5; // Increased damage for critical hits
      }
	  if (attacker.hasTag("sword")) {
	  caldamage = caldamage + 10
	  }
      // Apply the calculated damage
	  if (!hitEntity.hasTag("hit")) {
      hitEntity.applyDamage(caldamage, {
        cause: "none",
        source: attacker,
      });
	  hitEntity.addTag("hit");
	  system.runTimeout(() => {
                hitEntity.removeTag("hit");        
      }, 10);
	  }
    }
  }
}




const itemList = [
  "bf:wooden_axe",
  "bf:wooden_sword",
  "bf:stone_axe",
  "bf:stone_sword",
  "bf:iron_axe",
  "bf:iron_sword",
  "bf:golden_axe",
  "bf:golden_sword",
  "bf:diamond_axe",
  "bf:diamond_sword",
  "bf:netherite_axe",
  "bf:netherite_sword"
];

function itemDurabilityDecrease(itemStack, player, decreaseAmount) {
  const durability = itemStack.getComponent("durability");

  if (durability) {
    // Decrease durability by the specified amount
    durability.damage += decreaseAmount;

    // If durability reaches or exceeds maxDurability, remove the item
    if (durability.damage >= durability.maxDurability) {
      const inv = player.getComponent("inventory").container;
      inv.setItem(player.selectedSlotIndex, undefined); // Remove the item from inventory
      world.playSound("random.break", player.location); // Play break sound
      return null; // Indicate the item is broken
    }
  }
  return itemStack; // Return the modified item if it hasn’t broken
}

world.afterEvents.playerBreakBlock.subscribe((eventData) => {
  const { player, itemStackBeforeBreak } = eventData;
    if (!itemStackBeforeBreak) {return}
  // Use includes() to check if the item type is in the list of items
  if (itemList.includes(itemStackBeforeBreak.typeId)) {
    const newItem = itemDurabilityDecrease(itemStackBeforeBreak, player, 1);
    const inv = player.getComponent("inventory").container;
    
    if (newItem) {
      // Only set the item back if it's not broken
      inv.setItem(player.selectedSlotIndex, newItem);
    }
  }
});


world.afterEvents.entityDie.subscribe((event) => {
    const { deadEntity } = event;

    if (deadEntity.typeId === "minecraft:player") {
        const killer = recentHits.get(deadEntity.id);

        if (killer && killer.typeId === "minecraft:player") {
            // Custom death message when killed by another player
            const deathMessage = `§l${deadEntity.name} was slain by ${killer.name}`;
            world.sendMessage(deathMessage);
        }

        // Clean up recent hits for this player
        recentHits.delete(deadEntity.id);
    }
});