import { world, system, TicksPerSecond } from '@minecraft/server';
import { ModalFormData } from "@minecraft/server-ui";



// Dynamic properties to toggle
let dynamicProperties = [
    { id: "satdis", name: "SATURATION DISPLAY", value: true },
    { id: "sat", name: "JAVA SATURATION", value: true },
    { id: "web", name: "JAVA WEBS", value: true },
    { id: "old_web", name: "OLD WEBS", value: false },
    { id: "door", name: "JAVA DOORS", value: true },
    { id: "swap", name: "TOTEM SWAP", value: true },
	{ id: "dupe", name: "STRING DUPER", value: false }
];

// Helper function to fetch the dynamic property from the world or the local array
function dp(player, { id }) {
    // Fetch the dynamic property from the world
    const worldValue = world.getDynamicProperty(id);
    
    // If the world property is undefined, fallback to the local value
    if (worldValue === undefined) {
        // Find and return the local value from the array
        const localValue = dynamicProperties.find(property => property.id === id);
        return localValue ? localValue.value : false; // Default to false if not found
    }

    return worldValue;
}

function showItemForm(player) {
    // Create the form
    const form = new ModalFormData().title("BLURRYS PVP CONFIG");

    // Add a toggle for each dynamic property, fetching the current value from the world
    dynamicProperties.forEach(property => {
        form.toggle({ translate: property.name }, dp(player, { id: property.id }) ?? false);
    });

    // Show the form to the player
    form.show(player).then((response) => {
        if (response.canceled) {
            player.sendMessage("Configuration canceled.");
            return;
        }

        // Update the dynamic properties based on the form values
        dynamicProperties.forEach((property, index) => {
            const isChecked = response.formValues[index]; // true if checked, false if not
            property.value = isChecked; // Update the value in the array

            // Update the world dynamic property
            world.setDynamicProperty(property.id, isChecked);
        });
        
        player.sendMessage(`Config Saved`);
    });
}


function gethealth(entity) {
    // Retrieve the haste effect from the entity
    const healthEffect = entity.getEffect("minecraft:health_boost");

    // Check if the haste effect is present
    if (!healthEffect) {
        return 0; // No haste effect applied
    }

    // Retrieve the amplifier (Haste I = amplifier 0, Haste II = amplifier 1, etc.)
    const amplifier = healthEffect.amplifier;

    // Calculate the speed bonus using a mathematical formula
    const health = 4 * (amplifier + 1); // +1 because amplifier 0 is Haste I, etc.

    // Return the calculated haste speed
    return health;
}

// Listen for script events
system.afterEvents.scriptEventReceive.subscribe((event) => {
    const { id, sourceEntity } = event;

    // Check if the script event ID matches
    if (id === "bf:config") {
        showItemForm(sourceEntity)
	}
});

// Handles effects for golden apples
world.afterEvents.itemCompleteUse.subscribe(event => {
  try {
    let player = event.source;
    let item = event.itemStack;

    if (item.typeId === "minecraft:golden_apple") {
      player.runCommandAsync("effect @s absorption 0 0");
      player.runCommandAsync("effect @s absorption 120 0 true");
    }
  } catch (error) {
  }
});

// Food items and their saturation durations
const foods = [
    { item: "minecraft:apple", duration: 1.2 },
    { item: "minecraft:baked_potato", duration: 3 },
    { item: "minecraft:beetroot", duration: 0.5 },
    { item: "minecraft:beetroot_soup", duration: 3.6 },
    { item: "minecraft:bread", duration: 3 },
    { item: "minecraft:cake", duration: 0.2 },
    { item: "minecraft:carrot", duration: 1.8 },
    { item: "minecraft:chorus_fruit", duration: 1.2 },
    { item: "minecraft:cooked_chicken", duration: 3.6 },
    { item: "minecraft:cooked_cod", duration: 3 },
    { item: "minecraft:cooked_mutton", duration: 4.8 },
    { item: "minecraft:cooked_porkchop", duration: 6.4 },
    { item: "minecraft:cooked_rabbit", duration: 3 },
    { item: "minecraft:cooked_salmon", duration: 4.8 },
    { item: "minecraft:cookie", duration: 0.2 },
    { item: "minecraft:dried_kelp", duration: 0.1 },
    { item: "minecraft:enchanted_golden_apple", duration: 4 },
    { item: "minecraft:golden_apple", duration: 4 },
    { item: "minecraft:glow_berries", duration: 0.2 },
    { item: "minecraft:golden_carrot", duration: 7.2 },
    { item: "minecraft:honey_bottle", duration: 0.6 },
    { item: "minecraft:melon_slice", duration: 0.6 },
    { item: "minecraft:mushroom_stew", duration: 3.6 },
    { item: "minecraft:poisonous_potato", duration: 0.6 },
    { item: "minecraft:potato", duration: 0.3 },
    { item: "minecraft:pufferfish", duration: 0.1 },
    { item: "minecraft:pumpkin_pie", duration: 2.4 },
    { item: "minecraft:rabbit_stew", duration: 6 },
    { item: "minecraft:beef", duration: 0.9 },
    { item: "minecraft:chicken", duration: 0.6 },
    { item: "minecraft:cod", duration: 0.2 },
    { item: "minecraft:mutton", duration: 0.6 },
    { item: "minecraft:porkchop", duration: 0.9 },
    { item: "minecraft:rabbit", duration: 0.9 },
    { item: "minecraft:salmon", duration: 0.2 },
    { item: "minecraft:rotten_flesh", duration: 0.4 },
    { item: "minecraft:spider_eye", duration: 1.6 },
    { item: "minecraft:cooked_beef", duration: 6.4 },
    { item: "minecraft:suspicious_stew", duration: 3.6 },
    { item: "minecraft:sweet_berries", duration: 0.6 },
    { item: "minecraft:tropical_fish", duration: 0.1 }
];

// Function to apply saturation to players
function applySaturation(player, duration) {
  let saturationTime = player.getDynamicProperty("saturationTime") || 0;
  
  // Add the new duration to the existing saturation time
  saturationTime += duration;

  // Cap saturationTime at 10
  if (saturationTime > 10) {
    saturationTime = 10;
  }

  // Update the player's dynamic property
  player.setDynamicProperty("saturationTime", saturationTime);

  // Add the tag to indicate the player is affected
  player.addTag("dave:onSaturationEffect");
}


// Handle food consumption and apply saturation
world.afterEvents.itemCompleteUse.subscribe(({ itemStack, source }) => {
  foods.forEach(food => {
    if (itemStack.typeId === food.item) {
      applySaturation(source, food.duration);
    }
  });
});

// Manage saturation countdown
system.runInterval(() => {
  world.getAllPlayers().forEach(player => {
	const isSaturationOn = world.getDynamicProperty("sat");
    if (player.hasTag("dave:onSaturationEffect") && isSaturationOn) {
      let saturationTime = player.getDynamicProperty("saturationTime") || 0;
		
      if (saturationTime > 0) {
        let currentHealth = player.getComponent("minecraft:health").currentValue;
        if (currentHealth < 20 + gethealth(player)) {
		  saturationTime -= 1;
		  player.setDynamicProperty("saturationTime", saturationTime);
          player.addEffect("regeneration", 1 * TicksPerSecond, {
            amplifier: 3,
            showParticles: false,
          });
          player.addEffect("hunger", 1 * TicksPerSecond, {
            amplifier: 110,
            showParticles: false,
          });
		  system.runTimeout(() => {
                player.runCommandAsync(`effect @s clear regeneration`);
				player.runCommandAsync(`effect @s clear hunger`);
            }, 15);
        }
      } else {
        // Clear saturation effects when time runs out
        player.setDynamicProperty("saturationTime", 0);
        player.removeTag("dave:onSaturationEffect");

        // Log when saturation ends
      }
	  const isSaturationDisplayOn = world.getDynamicProperty("satdis");
	  if (isSaturationDisplayOn) {
		if (saturationTime > 0) {
		player.onScreenDisplay.setActionBar(`§7{||||||||||||||||§6}`);
		}
		if (saturationTime > 1) {
		player.onScreenDisplay.setActionBar(`§7{||||||||||||||§6||}`);
		}
		if (saturationTime > 2) {
		player.onScreenDisplay.setActionBar(`§7{||||||||||||§6||||}`);
		}
		if (saturationTime > 3) {
		player.onScreenDisplay.setActionBar(`§7{||||||||||§6||||||}`);
		}
		if (saturationTime > 4) {
		player.onScreenDisplay.setActionBar(`§7{||||||||§6||||||||}`);
		}
		if (saturationTime > 5) {
		player.onScreenDisplay.setActionBar(`§7{||||||§6||||||||||}`);
		}
		if (saturationTime > 6) {
		player.onScreenDisplay.setActionBar(`§7{||||§6||||||||||||}`);
		}
		if (saturationTime > 7) {
		player.onScreenDisplay.setActionBar(`§7{||§6||||||||||||||}`);
		}
		if (saturationTime > 8) {
		player.onScreenDisplay.setActionBar(`§7{§6||||||||||||||||}`);
		}
		if (saturationTime > 9) {
		player.onScreenDisplay.setActionBar(`§7§6{||||||||||||||||}`);
		}
    }
	}
  });
}, 20); // Runs every 20 ticks (1 second)

world.afterEvents.worldInitialize.subscribe(() => {
    system.runTimeout(() => {
		if (world.getDynamicProperty("satdis") === undefined) {
			world.setDynamicProperty("satdis", true); // Default to true (ON)
		}
		if (world.getDynamicProperty("sat") === undefined) {
			world.setDynamicProperty("sat", true); // Default to true (ON)
		}
		if (world.getDynamicProperty("web") === undefined) {
			world.setDynamicProperty("web", true); // Default to true (ON)
		}
		if (world.getDynamicProperty("door") === undefined) {
			world.setDynamicProperty("door", true); // Default to true (ON)
		}
		if (world.getDynamicProperty("swap") === undefined) {
			world.setDynamicProperty("swap", true); // Default to true (ON)
		}
		if (world.getDynamicProperty("old_web") === undefined) {
			world.setDynamicProperty("old_web", false); // Default to true (ON)
		}
		if (world.getDynamicProperty("dupe") === undefined) {
			world.setDynamicProperty("dupe", false); // Default to true (ON)
		}
        world.sendMessage("§6§lUse §f/scriptevent bf:config §6For config");
    }, 80); // 20 ticks = 1 second
});